use strict;

use DE_EPAGES::Amazon::API::AmazonTransaction;
use DE_EPAGES::Amazon::API::DTO::Address;
use DE_EPAGES::Amazon::API::DTO::Component;
use DE_EPAGES::Amazon::API::DTO::Item;
use DE_EPAGES::Amazon::API::DTO::OrderReport;
use DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler;
use DE_EPAGES::Amazon::API::SOAP::GetOrderDocumentResponseHandler;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent RunOnDirectory );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp UsePackage );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadClassByAlias
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS TAX_MODEL_NET );
use File::Basename;
use SOAP::Lite;
use Test::More;

my $ROOTPATH = '/Test';

#  DE_EPAGES::Amazon::API::SOAP::GetOrderDocumentResponseHandler or
#  DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler
my $HANDLERPACKAGE = 'DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler';

## no critic (ProhibitMagicNumbers)

####### test data ############
sub getNewOrderData {    ## no critic (ProhibitFunctionSize)
    my ( $BuyerAddress, $FulfillmentAddress, $TestNo ) = @_;
    my $O = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
    $O->AmazonOrderID('302-6636987-0853930-'.$TestNo);
    $O->AmazonSessionID('302-6636987-0853930-'.$TestNo);
    $O->OrderDate('2009-06-30T13:17:49+00:00');
    $O->ReportNumber('922629424');
    $O->BuyerName('Hansi Müller');
    $O->BuyerEmailAddress('p69XXXXXX5hp@marketplace.amazon.de');

    $O->Currency('EUR');
    $O->FulfillmentAddress('Lieferstr. 553');
    $O->FulfillmentMethod('Ship');
    $O->FulfillmentServiceLevel('Standard');

    $O->BuyerAddress($BuyerAddress);
    $O->FulfillmentAddress($FulfillmentAddress);

    my $I = DE_EPAGES::Amazon::API::DTO::Item->new;
    $I->AmazonOrderItemCode('05991383163091');
    $I->ProductTaxCode('PTC_PRODUCT_TAXABLE_A');
    $I->Title('Campingaz CV470 Ventilgaskartusche');
    $I->Quantity('2');
    $I->SKU('cg_0101104470');

    my $P = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P->Amount(1.55);
    $P->Type('Principal');

    my $P2 = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P2->Amount(3.55);
    $P2->Type('Shipping');

    $I->ItemPriceArray($P);
    $I->ItemPriceArray($P2);

    $O->ItemArray($I);

    return $O;
}

## no critic (ProhibitUnusedPrivateSubroutines)

#Addresstypes like mentioned in #EPG-27659
sub _getTestDataGermany1 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('ePages GmbH');
    $B->AddressFieldTwo('Heinrich-Heine-Str. 1');
    $B->City('Jena');
    $B->StateOrRegion('Jena');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');


    return [getNewOrderData( $B, $B, 'DE1' )] ;

}

sub _getTestDataNonGermany1 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('ePages GmbH');
    $B->AddressFieldTwo('Heinrich-Heine-Str. 1');
    $B->City('Jena');
    $B->StateOrRegion('Jena');
    $B->PostalCode('20222');
    $B->CountryCode('AT');
    $B->PhoneNumber('+49 40 123 123432');

    return [getNewOrderData( $B, $B, 'AT1' )] ;

}


sub _getTestDataGermany2 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('Heinrich-Heine-Str. 1');
    $B->AddressFieldTwo();
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    return [getNewOrderData( $B, $B, 'DE2' )] ;

}

sub _getTestDataNonGermany2 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('Heinrich-Heine-Str. 3');
    $B->AddressFieldTwo();
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('AT');
    $B->PhoneNumber('+49 40 123 123432');

    return [getNewOrderData( $B, $B, 'AT2' )] ;

}


sub _getTestDataGermany3 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('1234');
    $B->AddressFieldTwo('Packstation 1234');
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    return [getNewOrderData( $B, $B, 'DE3' )] ;

}


sub _getTestDataNonGermany3 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('1234');
    $B->AddressFieldTwo('Packstation 1234');
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('AT');
    $B->PhoneNumber('+49 40 123 123432');

    return [getNewOrderData( $B, $B, 'AT3' )] ;

}

sub _getTestDataGermany4 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('ePages GmbH');
    $B->AddressFieldTwo('Heinrich-Heine-Str. 1');
    $B->City('Jena');
    $B->StateOrRegion('Jena');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

   my $F = DE_EPAGES::Amazon::API::DTO::Address->new;
    $F->Name('Hansi Müller');
    $F->AddressFieldOne('ePages GmbH');
    $F->AddressFieldTwo('Heinrich-Heine-Str. 1');
    $F->City('Hamburg');
    $F->StateOrRegion('Hamburg');
    $F->PostalCode('20222');
    $F->CountryCode('AT');
    $F->PhoneNumber('+49 40 123 123432');


    return [getNewOrderData( $B, $F, 'DE4' )] ;

}



sub _getTestDataNonGermany4 { ## no critic (ProhibitFunctionSize)
    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('ePages GmbH');
    $B->AddressFieldTwo('Heinrich-Heine-Str. 1');
    $B->City('Jena');
    $B->StateOrRegion('Jena');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    my $orderData = getNewOrderData( $B, $B, 'AT4' );
    $orderData->BuyerEmailAddress('p69XXXXXX5hp@marketplace.amazon.at');

    return [$orderData] ;

}



sub _getTestDataSwitzerland {    ## no critic (ProhibitFunctionSize)
    my $O = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
    $O->AmazonOrderID('302-6636987-0853931');
    $O->AmazonSessionID('302-6636987-0853931');
    $O->OrderDate('2009-06-30T13:17:50+00:00');
    $O->ReportNumber('922629425');

    $O->BuyerEmailAddress('buyer@example.ch');
    $O->BuyerName('Albert Schweitzer');

    $O->Currency('EUR');
    $O->FulfillmentAddress('Lieferstr. 553');
    $O->FulfillmentMethod('Ship');
    $O->FulfillmentServiceLevel('Standard');

    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Albert Schweitzer');
    $B->AddressFieldOne('Kaeufergasse 123');
    $B->AddressFieldTwo();
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    $O->BuyerAddress($B);

    my $F = DE_EPAGES::Amazon::API::DTO::Address->new;
    $F->Name('Albert Schweitzer');
    $F->AddressFieldOne('Lieferstr. 14');
    $F->AddressFieldTwo('c/o Hermine Meier');
    $F->City('Genf');
    $F->StateOrRegion('Genf');
    $F->PostalCode('4321');
    $F->CountryCode('CH');
    $F->PhoneNumber('+41 61 123456');

    $O->FulfillmentAddress($F);

    my $I = DE_EPAGES::Amazon::API::DTO::Item->new;
    $I->AmazonOrderItemCode('05991383163091');
    $I->ProductTaxCode('PTC_PRODUCT_TAXABLE_A');
    $I->Title('Campingaz CV470 Ventilgaskartusche - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360 - This is a veeeeeery long title to test AD-3360');
    $I->Quantity('2');
    $I->SKU('cg_0101104470');

    my $P = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P->Amount(1.55);
    $P->Type('Principal');
    my $P2 = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P2->Amount(3.55);
    $P2->Type('Shipping');

    $I->ItemPriceArray($P);
    $I->ItemPriceArray($P2);

    $O->ItemArray($I);

    return [$O];
}

sub _getTestDataFromXML {
    my ($filename) = @_;
    my $Parser     = SOAP::Deserializer->new;
    my $SOM        = $Parser->deserialize( ${ GetFileContent($filename) } );
    UsePackage($HANDLERPACKAGE);
    my $Handler    = $HANDLERPACKAGE->new();
    $Handler->identifier('12345');

    # we are testing the internal handler method.
    $Handler->_handleOrderReport($SOM);    ## no critic (ProhibitPrivateSubroutineUse)
    my $aTree = $Handler->Result();
    return $aTree;
}
####### /test data ############

####### test helper ############
sub _testOrderImport {
    my ( $cTestData, $hRefValues ) = @_;
    my $Shop      = LoadObjectByPath($ROOTPATH)->child('TestShopAmazon');
    my $Merchant  = $Shop->folder('PortalConfigs')->child('AmazonTestConfig');
    my $aTestData = $cTestData->();

    foreach my $testData (@$aTestData) {

        # prepare stock test
        my $Product  = $Shop->folder('Products')->child('cg_0101104470');
        my $OldStock = $Product->get('StockLevel');

        my $AmazonTransaction = DE_EPAGES::Amazon::API::AmazonTransaction->new;
        my $cInsertTransSub   = sub {
            $AmazonTransaction->insert( $Merchant, [$testData] );
        };

        if ( !$hRefValues->{'StockReduce'} ) {
            BlockWarnings( $cInsertTransSub, [qw(NoProductForSKU)] );
        }
        else {
            &$cInsertTransSub;
        }

        my $hRefOrderValues = $hRefValues->{ $testData->{AmazonOrderID} };
        my $TestName        = $hRefOrderValues->{'TestName'};
        my $dbtlist
          = LoadClassByAlias('AmazonTransaction')->getObjects( 'ByExternalOrderID', [ $testData->{AmazonOrderID} ] );
        is( scalar @$dbtlist, 1, $TestName . '1 transaction generated' );

        # search order..
        my $matchingLineItems = $AmazonTransaction->getImportedLineItems();
        my $lineItem          = pop @$matchingLineItems;

        # check amounts
        cmp_ok( $lineItem->get('LineItemPrice'), '==', $hRefOrderValues->{'ProductPrice'},
            $TestName . 'product price' );

        my $NewStock;
        $NewStock = $Product->get('StockLevel');
        is(
            $NewStock,
            ( $OldStock - $hRefOrderValues->{'StockReduce'} ),
            $TestName . 'stock before import: ' . $OldStock . ' - after import ' . $NewStock
        );

        my $dbcontainer = $lineItem->get('LineItemContainer');
        my $dborder     = $dbcontainer->parent;
        ok( $dborder->get('PaidOn'),       $TestName . 'order is paid' );
        ok( $dborder->get('AmazonConfig'), $TestName . 'order has AmazonConfig' );

        # check lineitempayment amount
        is( $dbcontainer->get('Payment')->class->alias,
            'LineItemPayment', $TestName . 'Switch to base class LineItemPayment' );

        # check billing address
        my $dbbillingAddress   = $dborder->get('BillingAddress');
        my $testbillingAddress = $testData->BuyerAddress;

        is( $dbbillingAddress->get('City'),    $testbillingAddress->City,            $TestName . 'billing city' );
        is( $dbbillingAddress->get('State'),   $testbillingAddress->StateOrRegion,   $TestName . 'billing state' );
        is( $dbbillingAddress->get('Zipcode'), $testbillingAddress->PostalCode,      $TestName . 'billing plz' );
        is( $dbbillingAddress->get('EMail'),   $testData->BuyerEmailAddress,         $TestName . 'billing email'.$dbbillingAddress->get('EMail'));

        if( $hRefOrderValues->{'AddressChangeBilling'} ) {
            is( $dbbillingAddress->get('Street'),  $testbillingAddress->AddressFieldTwo, $TestName . 'billing street (changed)' );
            is( $dbbillingAddress->get('Street2'), undef, $TestName . 'billing street 2 (changed)' );
            is( $dbbillingAddress->get('Company'), $testbillingAddress->AddressFieldOne, $TestName . 'billing company (changed) ' );
        } else {
            is( $dbbillingAddress->get('Street'),  $testbillingAddress->AddressFieldOne, $TestName . 'billing street');
            is( $dbbillingAddress->get('Street2'), $testbillingAddress->AddressFieldTwo, $TestName . 'billing street 2');
            is( $dbbillingAddress->get('Company'), undef, $TestName . 'billing company' );
        }


        # check fulfillment address
        my $dbfulfillmentAddress   = $dborder->get('ShippingAddress');
        my $testfulfillmentAddress = $testData->FulfillmentAddress;

        is( $dbfulfillmentAddress->get('City'), $testfulfillmentAddress->City, $TestName . 'shipping city' );
        is( $dbfulfillmentAddress->get('State'), $testfulfillmentAddress->StateOrRegion, $TestName . 'shipping state' );
        is( $dbfulfillmentAddress->get('Zipcode'), $testfulfillmentAddress->PostalCode, $TestName . 'shipping plz' );

        if( $hRefOrderValues->{'AddressChangeShipping'} ) {
            is(
                $dbfulfillmentAddress->get('Street'),
                $testfulfillmentAddress->AddressFieldTwo,
                $TestName . 'shipping street (changed)'
            );
            is(
                $dbfulfillmentAddress->get('Street2'),
                undef,
                $TestName . 'shipping street 2 (changed)'
            );
            is(
                $dbfulfillmentAddress->get('Company'),
                $testfulfillmentAddress->AddressFieldOne,
                $TestName . 'shipping company (changed)'
            );
        } else {
            is(
                $dbfulfillmentAddress->get('Street'),
                $testfulfillmentAddress->AddressFieldOne,
                $TestName . 'shipping street'
            );
            is(
                $dbfulfillmentAddress->get('Street2'),
                $testfulfillmentAddress->AddressFieldTwo,
                $TestName . 'shipping street 2'
            );
            is(
                $dbfulfillmentAddress->get('Company'),
                undef,
                $TestName . 'shipping company'
            );


        }

        # check reference values
        cmp_ok( $dbcontainer->get('GrandTotal'), '==', $hRefOrderValues->{'GrandTotal'}, $TestName . 'grand total' );

        # total tax is a calculated float, so we have to compare using fcmp
        is( fcmp( $dbcontainer->get('TotalTax'), $hRefOrderValues->{'TotalTax'} ), 0, $TestName . 'total tax' )
          or note( 'TotalTax got / expected: '
              . sprintf( '%.16f / %.16f', $dbcontainer->get('TotalTax'), $hRefOrderValues->{'TotalTax'} ) );
        cmp_ok(
            $dbcontainer->get('Shipping')->get('LineItemPrice'),
            '==',
            $hRefOrderValues->{'ShippingPrice'},
            $TestName . 'shipping price'
        );
        is( $dbcontainer->get('TaxArea')->alias, $hRefOrderValues->{'TaxArea'},    $TestName . 'tax area' );
        is( $dbcontainer->get('TaxModel'),       $hRefOrderValues->{'TaxModel'},   $TestName . 'tax model' );
        is( $dbcontainer->get('CurrencyID'),     $hRefOrderValues->{'CurrencyID'}, $TestName . 'currency' );
    }
}
####### /test helper ############

####### tests ############
sub testPromo1 {
    _testOrderImport(
        sub { return _getTestDataFromXML('orderreport_promo1.xml') },
        {
            '028-8438651-3311525' => {
                TestName      => 'PROMO1: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 2.57,
                GrandTotal    => 16.10,
                ProductPrice  => 12.60,
                ShippingPrice => 3.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => 0,
                AddressChangeShipping => 0,
            },
        }
    );
}

sub testPromo2 {
    _testOrderImport(
        sub { return _getTestDataFromXML('orderreport_promo2.xml') },
        {
            '028-5256199-7777953' => {
                TestName      => 'PROMO2: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 1.66,
                GrandTotal    => 10.40,
                ProductPrice  => 7.90,
                ShippingPrice => 2.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => 1,
                AddressChangeShipping => 0,
            },
        }
    );
}

sub testDataGermany {
    my ( $hRefValues ) = @_;
    _testOrderImport(
        \&{'_getTestDataGermany'.$hRefValues->{'TestNo'}},
        {
            '302-6636987-0853930-DE'.$hRefValues->{'TestNo'} => {
                TestName      => 'DE'.$hRefValues->{'TestNo'}.': ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                GrandTotal    => 5.10,
                TotalTax      => 0.81,
                ShippingPrice => 3.55,
                ProductPrice  => 1.55,
                StockReduce   => 2,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => $hRefValues->{'AddressChangeBilling'},
                AddressChangeShipping => $hRefValues->{'AddressChangeShipping'},
            },
        }
    );

}


sub testDataNonGermany {
    my ( $hRefValues ) = @_;
    _testOrderImport(
        \&{'_getTestDataNonGermany'.$hRefValues->{'TestNo'}},
        {
            '302-6636987-0853930-AT'.$hRefValues->{'TestNo'} => {
                TestName      => 'AT'.$hRefValues->{'TestNo'}.': ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                GrandTotal    => 5.10,
                TotalTax      => 0.81,
                ShippingPrice => 3.55,
                ProductPrice  => 1.55,
                StockReduce   => 2,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => $hRefValues->{'AddressChangeBilling'},
                AddressChangeShipping => $hRefValues->{'AddressChangeShipping'},
            },
        }
    );

}




sub testDataSwitzerland {
    _testOrderImport(
        \&_getTestDataSwitzerland,
        {
            '302-6636987-0853931' => {
                TestName      => 'CH: ',
                TaxArea       => 'non EU',
                TaxModel      => TAX_MODEL_NET,
                GrandTotal    => 5.10,
                TotalTax      => 0.0,
                ShippingPrice => 3.55,
                ProductPrice  => 1.55,
                StockReduce   => 2,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => 0,
                AddressChangeShipping => 0,
            },
        }
    );
}

sub testMultiCurrency {
    _testOrderImport(
        sub { return _getTestDataFromXML('orderreport_multicurrency.xml') },
        {
            '326-4366989-9695541' => {
                TestName      => 'MULTI1: ',
                TaxArea       => 'non EU',
                TaxModel      => TAX_MODEL_NET,
                GrandTotal    => 16.85,
                TotalTax      => 0.00,
                ShippingPrice => 4.50,
                ProductPrice  => 12.35,
                CurrencyID    => 'GBP',
                AddressChangeBilling  => 0,
                AddressChangeShipping => 0,
            },
            '128-5256199-7777953' => {
                TestName      => 'MULTI2: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 1.66,
                GrandTotal    => 10.40,
                ProductPrice  => 7.90,
                ShippingPrice => 2.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => 0,
                AddressChangeShipping => 0,
            },
        }
    );
}
sub testMultiCurrency2 {
    _testOrderImport(
        sub { return _getTestDataFromXML('orderreport_multicurrency2.xml') },
        {
            '326-4366989-9695542' => {
                TestName      => 'MULTI2_1: ',
                TaxArea       => 'non EU',
                TaxModel      => TAX_MODEL_NET,
                GrandTotal    => 16.85,
                TotalTax      => 0.00,
                ShippingPrice => 4.50,
                ProductPrice  => 12.35,
                CurrencyID    => 'GBP',
                AddressChangeBilling  => 0,
                AddressChangeShipping => 0,
            },
            '128-5256199-7777955' => {
                TestName      => 'MULTI2_2: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 1.66,
                GrandTotal    => 10.40,
                ProductPrice  => 7.90,
                ShippingPrice => 2.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
                AddressChangeBilling  => 0,
                AddressChangeShipping => 0,
            },
        }
    );
}
####### /tests ############

####### Main ############
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # initialization
            ok( Init(), 'initialization' );

            # tests
            eval {
                # manual data structure

                #Addresstypes like mentioned in #EPG-27659
                #German Address with Company and Street
                testDataGermany({TestNo => 1, AddressChangeBilling => 1, AddressChangeShipping=> 1});
                # German Address with Street only
                testDataGermany({TestNo => 2, AddressChangeBilling => 0, AddressChangeShipping=> 0});
                # German Address with Packstation
                testDataGermany({TestNo => 3, AddressChangeBilling => 0, AddressChangeShipping=> 0});
                # Address with Company and Street, but billing in Germany and shipping is not Germany
                testDataGermany({TestNo => 4, AddressChangeBilling => 1, AddressChangeShipping=> 0});

                testDataNonGermany({TestNo => 1, AddressChangeBilling => 0, AddressChangeShipping=> 0});
                testDataNonGermany({TestNo => 2, AddressChangeBilling => 0, AddressChangeShipping=> 0});
                testDataNonGermany({TestNo => 3, AddressChangeBilling => 0, AddressChangeShipping=> 0});
                testDataNonGermany({TestNo => 4, AddressChangeBilling => 0, AddressChangeShipping=> 0});


                testDataSwitzerland();

                # # $HANDLERPACKAGE = 'DE_EPAGES::Amazon::API::SOAP::GetOrderDocumentResponseHandler';
                testPromo1();
                testPromo2();
                testMultiCurrency();
                testMultiCurrency2();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'ERROR: ' . $Error->longMessage );
            }

            # cleanup
            ok( Cleanup(), 'cleanup' );
        }
    );
    done_testing;
    return;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    # cleanup AmazonTransactions if they are still in the system.
    foreach my $AmazonOrderID (qw (302-6636987-0853930 302-6636987-0853931 128-5256199-7777953 326-4366989-9695541 )) {
        my $dbtlist = LoadClassByAlias('AmazonTransaction')->getObjects( 'ByExternalOrderID', [$AmazonOrderID] );
        foreach my $dbt ( @{$dbtlist} ) {
            $dbt->delete;
        }
    }

    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => sub {
    RunOnDirectory(dirname (__FILE__), sub {
        Main();
    });
});

1;
