#========================================================================================
# package      DE_EPAGES::Core::API::Git
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides Git commands.
#----------------------------------------------------------------------------------------
# example      my $Git = DE_EPAGES::Core::API::Git->new();
#               $Git->CloneRepository($RepoUrl);
#========================================================================================
package DE_EPAGES::Core::API::Git;
use base qw( Exporter );

use strict;
use Git::Repository;
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::File qw (
    ExistsDirectory
    RunOnDirectory
);

our @EXPORT_OK = qw (
    CloneRepository
    GetPatchCartridge
    GitExportRepo
    Add
    Commit
    Checkout
    CreateBranch
    ExistsRemoteBranch
    ExistsLocalBranch
    Merge
    Push
    MapGitToPath
    ProjectUpdates4GitFingerprint
);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Git = DE_EPAGES::Core::API::Git->new($hOptions);
#----------------------------------------------------------------------------------------
# description  Create a Git object.
#----------------------------------------------------------------------------------------
# return       $Git | Git object | object
#========================================================================================
sub new {
    my $class = shift;
    my $self->{'Company'} = 'DE_EPAGES';
    $self->{'CartridgeRepo'} = $ENV{'EPAGES_CARTRIDGES'};
    return bless $self, $class;
}

#========================================================================================
# function     CloneRepository
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->CloneRepository($RepoUrl, $TargetDir, $aArguments);
#----------------------------------------------------------------------------------------
# description  Clone a Git Repository.
#----------------------------------------------------------------------------------------
# input        $RepoUrl | url of a GitHub repository | string
# input        $TargetDir | target directory to clone files to | string
# input        $aArguments | git clone arguments | ref.array.string
#========================================================================================
sub CloneRepository {
    my $self = shift;
    my ($RepoUrl, $TargetDir, $aArguments) = @_;

    die "\nmissing 'RepoUrl'" unless (defined $RepoUrl);
    Git::Repository->run( clone => @$aArguments => $RepoUrl => $TargetDir);
}

#========================================================================================
# function     GetPatchCartridge
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->GetPatchCartridge($Tag, $TargetDir);
#----------------------------------------------------------------------------------------
# description  Get an extracted archive of the Cartridge repository (or specified Cartridge)
#               for specified tag/branch.
#----------------------------------------------------------------------------------------
# input        $Version | version to create tmp folder in Patches/PatchTemp/release_$Version | string
# input        $BranchOrTag | branch to get latest commit point or specified tag point from | string
# input        $CartridgePath | path to cartridge, e.g. /srv/epages/eproot/Cartridges/FlexProvider | string
# input        $TargetDir | target directory to extract files to | string
#========================================================================================
sub GetPatchCartridge {
    my $self = shift;
    my ($Version, $BranchOrTag, $CartridgePath, $TargetDir) = @_;

    die "\nmissing target directory" unless (defined $TargetDir);
    die "\nmissing 'version' value" unless (defined $Version);
    die "\nmissing 'tag' or 'branch' to create archive from" unless (defined $BranchOrTag);

    $Version=~ s/\./_/g;
    my $TarFile = $Version.'.tar';

    $CartridgePath=~ s/\/srv\/epages\/eproot\/Cartridges\///;
    $self->GitExportRepo($BranchOrTag, $self->{'CartridgeRepo'}, $TargetDir, $TarFile, $CartridgePath);
}

#========================================================================================
# function     GitExportRepo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->GitExportRepo($BranchOrTag, $RepoTarget, $TargetDir, $TarFile, $CartridgePath);
#----------------------------------------------------------------------------------------
# description  Get an extracted archive of the Cartridge repository (or specified Cartridge)
#               for specified tag/branch.
#----------------------------------------------------------------------------------------
# input        $BranchOrTag | branch to get latest commit point or specified tag point from | string
# input        $RepoSource | folder where your .git folder is. Files will be exported from here. | string
# input        $TargetDir | target directory to extract files to | string
# input        $TarFile | name of the tar file | string
# input        $CartridgePath | path to cartridge, e.g. /srv/epages/eproot/Cartridges/FlexProvider | string
#========================================================================================
sub GitExportRepo {
    my $self = shift;
    my ($BranchOrTag, $RepoSource, $TargetDir, $TarFile, $CartridgePath) = @_;

    my $Archive = $TargetDir.'/CartridgesArchive.tar';
    $Archive = $TargetDir.'/'.$TarFile if (defined $TarFile);

    die "\nFolder '$RepoSource' does not exist" unless ExistsDirectory($RepoSource);

    RunOnDirectory(
        $RepoSource,
        sub {
            my $Git = $self->_createGitRepository();
            if ($CartridgePath) {
                $Git->run('archive', '--format=tar', "--output=$Archive", $BranchOrTag, $CartridgePath);
            } else {
                $Git->run('archive', '--format=tar', "--output=$Archive", $BranchOrTag);
            }
            #untar file
            RunOnDirectory(
                $TargetDir,
                sub {
                    SystemCommand("tar xf $Archive");
                    SystemCommand("rm $Archive");
                    return;
                }
                );
            return;
        }
        );
}

#========================================================================================
# function     Add
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->Add($FileOrDirectory);
#----------------------------------------------------------------------------------------
# description  Add a file or directory for the next commit.
#----------------------------------------------------------------------------------------
# input        $FileOrDirectory | file or directory to add for commit | string
# input        $aArguments | git add arguments | ref.array.string
#========================================================================================
sub Add {
    my $self = shift;
    my ($FileOrDirectory, $aArguments) = @_;

    my $Git = $self->_createGitRepository();
    $Git->run( add => @$aArguments => $FileOrDirectory);
}

#========================================================================================
# function     Remove
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->Remove($FileOrDirectory);
#----------------------------------------------------------------------------------------
# description  Remove a file or directory for the next commit.
#----------------------------------------------------------------------------------------
# input        $FileOrDirectory | file or directory to remove for commit | string
# input        $aArguments | git rm arguments | ref.array.string
#========================================================================================
sub Remove {
    my $self = shift;
    my ($FileOrDirectory, $aArguments) = @_;

    my $Git = $self->_createGitRepository();
    $Git->run( rm => @$aArguments => $FileOrDirectory);
}

#========================================================================================
# function     Commit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->Commit($CommitString, $aArguments);
#----------------------------------------------------------------------------------------
# description  Commit changes to current branch.
#----------------------------------------------------------------------------------------
# input        $CommitString | commit message | string
# input        $aArguments | git commit arguments,
#                             e.g. '-a' to directly add and commit a modified file | ref.array.string
#========================================================================================
sub Commit {
    my $self = shift;
    my ($CommitString, $aArguments) = @_;

    die "\nmissing commit message" unless (defined $CommitString);
    my $Git = $self->_createGitRepository();
    $Git->run( commit => @$aArguments => '-m', $CommitString);
}

#========================================================================================
# function     Checkout
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->Checkout($Branch, $aArguments);
#----------------------------------------------------------------------------------------
# description  Checkout a branch.
#----------------------------------------------------------------------------------------
# input        $Branch | name of the branch | string
# input        $aArguments | git checkout arguments | ref.array.string
#========================================================================================
sub Checkout {
    my $self = shift;
    my ($Branch, $aArguments) = @_;

    my $Git = $self->_createGitRepository();
    $Git->run( 'checkout' => @$aArguments => $Branch);
}

#========================================================================================
# function     CreateBranch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->CreateBranch($Branch, $aArguments);
#----------------------------------------------------------------------------------------
# description  Create a branch.
#----------------------------------------------------------------------------------------
# input        $Branch | name of the branch | string
# input        $aArguments | git checkout arguments | ref.array.string
#========================================================================================
sub CreateBranch {
    my $self = shift;
    my ($Branch, $aArguments) = @_;

    my $Git = $self->_createGitRepository();
    $Git->run( 'branch' => @$aArguments => $Branch);
}

#========================================================================================
# function     ExistsRemoteBranch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->ExistsRemoteBranch($Branch);
#----------------------------------------------------------------------------------------
# description  Check if a branch already exists on remote.
#----------------------------------------------------------------------------------------
# input        $Branch | name of the branch | string
# return       1, if branch already exists on remote, 0 otherwise | bool
#========================================================================================
sub ExistsRemoteBranch {
    my $self = shift;
    my ($Branch) = @_;

    my $Git = $self->_createGitRepository();
    my $RemoteBranches = $Git->run( 'branch' => '-r');
    if($RemoteBranches=~ /origin\/$Branch/ or
       $RemoteBranches=~ /upstream\/$Branch/ or
       $RemoteBranches=~ /$Branch/)
    {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     ExistsLocalBranch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->ExistsLocalBranch($Branch);
#----------------------------------------------------------------------------------------
# description  Check if a branch already exists local.
#----------------------------------------------------------------------------------------
# input        $Branch | name of the branch | string
# return       1, if branch already exists local, 0 otherwise | bool
#========================================================================================
sub ExistsLocalBranch {
    my $self = shift;
    my ($Branch) = @_;

    my $Git = $self->_createGitRepository();
    my $LocalBranches = $Git->run( 'branch' );
    if($LocalBranches=~ /$Branch/) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     Merge
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->Merge($MergeBranch, $FeatureBranch, $aArguments);
#----------------------------------------------------------------------------------------
# description  Merge changes from $FeatureBranch to $MergeBranch.
#----------------------------------------------------------------------------------------
# input        $MergeBranch | name of the branch to merge into | string
# input        $FeatureBranch | name of the branch to merge from | string
# input        $aArguments | git merge arguments | ref.array.string
#========================================================================================
sub Merge {
    my $self = shift;
    my ($MergeBranch, $FeatureBranch, $aArguments) = @_;

    $self->Checkout($MergeBranch);
    my $Git = $self->_createGitRepository();
    $Git->run( merge => @$aArguments => $FeatureBranch);
}

#========================================================================================
# function     Push
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->Push($Remote, $aArguments);
#----------------------------------------------------------------------------------------
# description  Push current changes (file or directory) to remote.
#----------------------------------------------------------------------------------------
# input        $Remote | remote the changes will be pushed to | string
# input        $Branch | branch to be pushed | string
# input        $aArguments | git push arguments | ref.array.string
#========================================================================================
sub Push {
    my $self = shift;
    my ($Remote, $Branch, $aArguments) = @_;

    print "\nmissing remote" unless (defined $Remote);
    my $Git = $self->_createGitRepository();
    $Git->run( 'push' => @$aArguments => $Remote, $Branch);
}

sub _createGitRepository() {
    my $self = shift;
    return Git::Repository->new();
}

#========================================================================================
# function     GetCommitId
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->GetCommitId($Commit, $aArguments);
#----------------------------------------------------------------------------------------
# description  Get the commit ID (SHA-1) of given commit (i.e. HEAD). Based on the git
#               command rev-parse.
#----------------------------------------------------------------------------------------
# input        $Commit | tag to identify a commit (i.e 'HEAD')| string
# input        $aArguments | git rev-parse arguments (i.e. '--short') | ref.array.string
#========================================================================================
sub GetCommitID {

    my $self = shift;
    my ($Commit, $aArguments) = @_;

    die "\nmissing commit" unless (defined $Commit);

    return Git::Repository->run( 'rev-parse' => @$aArguments => $Commit);
}

#========================================================================================
# function     MapGitToPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PathList = MapGitToPath($GitRepo, $File)
#----------------------------------------------------------------------------------------
# description  Returns hash with absolute paths in file system as key
#               and role as value for $File in $GitRepo.
#               $GitRepo must be one of following repositories:
#               <ul>
#                 <li>ePages-de/Cartridges
#                 <li>ePages-de/epages-perl
#                 <li>ePages-de/epages6-packages
#                 <li>ePages-de/node-binaries
#                 <li>ePages-de/node-epages6
#               </ul>
# example      {'/srv/epages/eproot/Cartridges/Cartridges.xml' => 'local'} = MapGitToPath('ePages-de/Cartridges', 'Cartridges.xml')
#----------------------------------------------------------------------------------------
# input        $GitRepo  | GIT repository | string
# input        $File     | file in GIT repository | string
# return       $PathList | hash with absolute paths as key and role as value, role is one of:
#                         <ul>
#                           <li>common
#                           <li>apache
#                           <li>appsrv
#                           <li>ascsrv
#                           <li>reqsrv
#                           <li>unity
#                           <li>am-yum
#                         </ul> | ref.hash
#========================================================================================
sub MapGitToPath {
    my ($GitRepo, $File) = @_;

    if ($GitRepo =~ m,(^|[:/])ePages-de/Cartridges(\.|$),i) {
        return _mapGitCartridgesToPath($File);
    } elsif ($GitRepo =~ m,(^|[:/])ePages-de/epages-perl(\.|$),i) {
        return _mapGitEpagesPerlToPath($File);
    } elsif ($GitRepo =~ m,(^|[:/])ePages-de/epages6-packages(\.|$),i) {
        return _mapGitEpages6PackagesToPath($File);
    } elsif ($GitRepo =~ m,(^|[:/])ePages-de/node-binaries(\.|$),i) {
        return _mapGitNodeBinariesToPath($File);
    } elsif ($GitRepo =~ m,(^|[:/])ePages-de/node-epages6(\.|$),i) {
        return _mapGitNodeEpages6ToPath($File);
    } else {
        Error("Unknown GIT repository: $GitRepo");
    }
}

sub _mapGitCartridgesToPath{
    my ($File) = @_;
    return $File =~ /^\./ ? { "/srv/epages/eproot/Cartridges/$File" => 'common' } : {};
}

sub _mapGitEpagesPerlToPath{
    my ($File) = @_;
    return $File =~ s,^data/,/, ? { $File => 'common' } : {};
}

sub _mapGitEpages6PackagesToPath{
    my ($File) = @_;
    ##no critic qw(ProhibitCascadingIfElse)
    if ( $File =~ s,pkg/epages-appsrv/data/,/,) {
        return { $File => 'appsrv' };
    } elsif ( $File =~ s,pkg/epages-ascsrv/data/,/,) {
        return { $File => 'ascsrv' };
    } elsif ( $File =~ s,pkg/epages-common/data/,/,) {
        return { $File => 'common' };
    } elsif ( $File =~ s,pkg/epages-intern-release/data/,/,) {
        return { $File => 'am-yum' };
    } elsif ( $File =~ s,pkg/epages-nodejs/data/,/,) {
        return { $File => 'common' };
    } elsif ( $File =~ s,pkg/epages-release/data/,/,) {
        return { $File => 'common' };
    } elsif ( $File =~ s,pkg/epages-reqsrv/data/,/,) {
        return { $File => 'reqsrv' };
    } elsif ( $File =~ s,pkg/epages-unity/data/,/,) {
        return { $File => 'unity' };
    } elsif ( $File =~ s,pkg/epages-webconf/data/,/,) {
        return { $File => 'apache' };
    } else {
        return {};
    }
    ##use critic
}

sub _mapGitNodeBinariesToPath{
    my ($File) = @_;
    return $File =~ s,^data/,/, ? { $File => 'common' } : {};
}

sub _mapGitNodeEpages6ToPath{
    my ($File) = @_;
    my $PathList = { "/srv/epages/eproot/Perl/bin/nodejs.d/lib/node_modules/node-epages6/$File" => 'common' };
    $PathList->{"/srv/epages/eproot/Perl/$File"} = 'common' if $File =~ m,^bin/,;
    return $PathList;
}

#========================================================================================
# function     ProjectUpdates4GitFingerprint
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProjectUpdates4GitFingerprint($GitRepo, $Branch, $Fingerprint, $System, $Roles, $Node, $Dir)
#----------------------------------------------------------------------------------------
# description  Changes hash $Fingerprint according to $System, $Roles, $Node, $Dir in $Branch of $GitRepo.
# example      ProjectUpdates4GitFingerprint($GitRepo, \%Fingerprint, 'live', [qw(common appsrv)],
#                                             'my.host.com', "~/my-git/repo");
#----------------------------------------------------------------------------------------
# input        $GitRepo | Git repository object | ref.object
# input        $Branch  | Git branch like 'origin/release/7.11.0' | string
# input        $Fingerprint | hash to change | ref.hash
# input        $Node   | hash with keys host, roles and system | ref.hash
# input        $Dir    | Git directory containing .git | string
# input        $Quiet  | no output if set | boolean
#========================================================================================
sub ProjectUpdates4GitFingerprint {
    my ($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose) = @_;

    _updateFingerprintRoles($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose);
    _updateFingerprintSystems($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose);
    _updateFingerprintNodes($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose);
}

sub _updateFingerprintRoles {
    my ($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose) = @_;

    return unless defined $Dir and defined $Node->{roles};

    foreach my $Role (@{$Node->{roles}}) {
        next unless -d "$Dir/Installation-Roles/$Role";
        print "$Dir: git ls-tree -r $Branch Installation-Roles/$Role\n" if $Verbose;
        foreach my $Line ($GitRepo->run( 'ls-tree' => '-r', $Branch, "Installation-Roles/$Role")) {
            my ($GitType, $Sha1, $File) = $Line =~ /^\s*\S+\s+(\S+)\s+(\S+)\s+(.*)/;
            next unless $GitType eq 'blob';
            $File =~ s,Installation-Roles/$Role/,/,;
            $Fingerprint->{$Role}{$File} = $Sha1;
        }
    }
}

sub _updateFingerprintSystems {
    my ($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose) = @_;

    return unless defined $Dir and defined $Node->{roles} and defined $Node->{system};

    foreach my $Role (@{$Node->{roles}}) {
        next unless -d "$Dir/Installation-Systems/$Node->{system}/$Role";
        print "$Dir: git ls-tree -r $Branch Installation-Systems/$Node->{system}/$Role\n" if $Verbose;
        foreach my $Line ($GitRepo->run( 'ls-tree' => '-r', $Branch, "Installation-Systems/$Node->{system}/$Role")) {
            my ($GitType, $Sha1, $File) = $Line =~ /^\s*\S+\s+(\S+)\s+(\S+)\s+(.*)/;
            next unless $GitType eq 'blob';
            $File =~ s,Installation-Systems/$Node->{system}/$Role/,/,;
            $Fingerprint->{$Role}{$File} = $Sha1;
        }
    }
}

sub _updateFingerprintNodes {
    my ($GitRepo, $Branch, $Fingerprint, $Node, $Dir, $Verbose) = @_;

    return unless defined $Dir and defined $Node->{system} and defined $Node->{host};
    return unless -d "$Dir/Installation-Nodes/$Node->{system}/$Node->{host}";

    print "$Dir: git ls-tree -r $Branch Installation-Nodes/$Node->{system}/$Node->{host}\n" if $Verbose;
    foreach my $Line ($GitRepo->run( 'ls-tree' => '-r', $Branch, "Installation-Nodes/$Node->{system}/$Node->{host}")) {
        my ($GitType, $Sha1, $File) = $Line =~ /^\s*\S+\s+(\S+)\s+(\S+)\s+(.*)/;
        next unless $GitType eq 'blob';
        $File =~ s,Installation-Nodes/$Node->{system}/$Node->{host}/,/,;
        $Fingerprint->{'common'}{$File} = $Sha1;
    }
}

1;
