#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::UI_MolliePayments
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::UI_MolliePayments;
use base DE_EPAGES::Core::API::Test;

use Test::Mockify;
use Test::Mockify::Sut;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use Test::Mockify::ReturnValue;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Mockify::Matcher qw ( String Object Number HashRef );

use strict;

use constant PREDEFINED_AMOUNT => '10.59';

sub test {
    my $self = shift;

    $self->test_MolliePaymentsRefunds_updateRefundAmountsByGetMolliePayment();
    $self->test_MolliePaymentsRefunds_doRefund();
    $self->test_MolliePaymentsRefunds_updateLineItemRefundAmounts();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_MolliePaymentsRefunds_updateRefundAmountsByGetMolliePayment {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => undef,
        'amountRemaining' => undef,
    };

    my $MollieRequestService = $self->_createPaymentsClient($hPaymentResponse);
    my $SUT = $self->_createSUT($MollieRequestService);

    my $hLineItemPaymentAttributes = {
        'Amount' => 10.5,
        'TransID' => 'tr_srwerw',
        'TransStatus' => 'paid',
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments($hLineItemPaymentAttributes);

    lives_ok(
        sub {$SUT->updateRefundAmountsByGetMolliePayment($LineItemPayment); },
        "$SubTestName - updateRefundAmountsByGetMolliePayment is called without errors"
    );

    ok(  WasCalled( $SUT, '_getMolliePayment' ),
                "$SubTestName - _getMolliePayment was called in updateRefundAmountsByGetMolliePayment."
    );

    ok(  WasCalled( $SUT, 'updateLineItemRefundAmounts' ),
                "$SubTestName - updateLineItemRefundAmounts was called in updateRefundAmountsByGetMolliePayment."
    );

    is_deeply(  GetParametersFromMockifyCall( $SUT, 'updateLineItemRefundAmounts'),
        [$LineItemPayment, $hPaymentResponse],
        "$SubTestName - updateLineItemRefundAmounts was called with correct arguments."
    );

    return;
}
sub test_MolliePaymentsRefunds_doRefund {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $RefundAmount = PREDEFINED_AMOUNT;

    my $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => {'value' => $RefundAmount},
        'amountRemaining' => {'value' => '35.99'},
    };

    my $hRefundResponse = {
        'id' => 'rfb_srwerw',
        'status' => 'pending',
        'amount' => $RefundAmount
    };

    my $MollieRequestService = $self->_createPaymentsClient($hPaymentResponse, $hRefundResponse);
    my $SUT = $self->_createSUT($MollieRequestService);

    my $hLineItemPaymentAttributes = {
        'Amount' => 20.60,
        'TransID' => 'tr_srwerw',
        'TransStatus' => 'paid',
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments($hLineItemPaymentAttributes);

    my $hActualResult = undef;
    lives_ok(
        sub {$hActualResult = $SUT->doRefund($LineItemPayment, PREDEFINED_AMOUNT); },
        "$SubTestName - doRefund is called without errors"
    );

    ok( WasCalled( $SUT, '_getMolliePayment' ),
                "$SubTestName - _getMolliePayment was called in doRefund."
    );

    ok( WasCalled( $SUT, 'updateLineItemRefundAmounts' ),
                "$SubTestName - updateLineItemRefundAmounts was called in doRefund."
    );

    is_deeply(  GetParametersFromMockifyCall( $SUT, 'updateLineItemRefundAmounts'),
        [$LineItemPayment, $hPaymentResponse],
        "$SubTestName - updateLineItemRefundAmounts was called with correct arguments."
    );

    is_deeply( $hActualResult,
        {
            'id' => 'rfb_srwerw',
            'paymentId' => 'tr_srwerw',
            'status' => 'pending',
            'amount' => $RefundAmount,
            'updatedRefundAmounts' => {
                'AmountRefunded' => PREDEFINED_AMOUNT,
                'AmountRemaining' => '10.01',
            }
        },
        "$SubTestName - doRefund returns the correct result."
    );

    

    return;
}

sub test_MolliePaymentsRefunds_updateLineItemRefundAmounts {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];


    my $SUT = $self->_createSUT('');

    my $hLineItemPaymentAttributes = {
        'Amount' => PREDEFINED_AMOUNT,
        'TransID' => 'tr_srwerw',
        'TransStatus' => 'paid',
    };
    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments($hLineItemPaymentAttributes);
    my $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => undef,
        'amountRemaining' => {'value' => '35.59'},
    };
    lives_ok(
        sub {$SUT->updateLineItemRefundAmounts($LineItemPayment,$hPaymentResponse); },
        "$SubTestName - updateLineItemRefundAmounts is called without errors"
    );

    is ($LineItemPayment->get('AmountRemaining'), PREDEFINED_AMOUNT,
        "$SubTestName - LineItemPayment AmountRemaining gets updated to 10.5"
    );

    $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => {'value' => '5.59'},
        'amountRemaining' => {'value' => '30.09'},
    };

    lives_ok(
        sub {$SUT->updateLineItemRefundAmounts($LineItemPayment,$hPaymentResponse); },
        "$SubTestName - updateLineItemRefundAmounts is called without errors"
    );

    is ($LineItemPayment->get('AmountRemaining'), '5.00',
        "$SubTestName - LineItemPayment AmountRemaining gets updated to 5"
    );

    $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => {'value' => '11.59'},
        'amountRemaining' => {'value' => '20.09'},
    };

    lives_ok(
        sub {$SUT->updateLineItemRefundAmounts($LineItemPayment,$hPaymentResponse); },
        "$SubTestName - updateLineItemRefundAmounts is called without errors"
    );

    is ($LineItemPayment->get('AmountRemaining'), '0.00',
        "$SubTestName - LineItemPayment with over refunding AmountRemaining gets updated to 0.00"
    );

    my $UpdatedAmount = 1235436.99;
    $LineItemPayment->set({'Amount' => $UpdatedAmount});
    $LineItemPayment->parent()->set({'GrandTotal' => $UpdatedAmount});
    $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => {'value' => '1235435.01'},
        'amountRemaining' => {'value' => '20.09'},
    };

    lives_ok(
        sub {$SUT->updateLineItemRefundAmounts($LineItemPayment,$hPaymentResponse); },
        "$SubTestName - updateLineItemRefundAmounts is called without errors"
    );

    is ($LineItemPayment->get('AmountRemaining'), '1.98',
        "$SubTestName - LineItemPayment AmountRemaining gets updated to 1.98"
    );

    $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => {'value' => '1235436.98'},
        'amountRemaining' => {'value' => '20.09'},
    };

    lives_ok(
        sub {$SUT->updateLineItemRefundAmounts($LineItemPayment,$hPaymentResponse); },
        "$SubTestName - updateLineItemRefundAmounts is called without errors"
    );

    is ($LineItemPayment->get('AmountRemaining'), '0.01',
        "$SubTestName - LineItemPayment AmountRemaining gets updated to 0.01"
    );

    $hPaymentResponse = {
        'id' => 'tr_srwerw',
        'status' => 'paid',
        'amountRefunded' => {'value' => '1235436.99'},
        'amountRemaining' => {'value' => '20.09'},
    };

    lives_ok(
        sub {$SUT->updateLineItemRefundAmounts($LineItemPayment,$hPaymentResponse); },
        "$SubTestName - updateLineItemRefundAmounts is caflled without errors"
    );

    is ($LineItemPayment->get('AmountRemaining'), '0.00',
        "$SubTestName - LineItemPayment AmountRemaining gets updated to 0.00"
    );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Service) = @_;

    my $Mockify = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService');

    $Mockify->mockImported('DE_EPAGES::Core::API::TestParameter', 'TestObject')->when(String(), Object(), String())->thenReturnUndef();
    $Mockify->mock( '_createMolliePaymentsClient')->when(Object())->thenReturn($Service);
    $Mockify->mock('_LogPayment')->whenAny()->thenReturnUndef();
    $Mockify->spy('_getMolliePayment')->when(
            Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'),
            Object('DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments')
    );
    $Mockify->spy('updateLineItemRefundAmounts')->when(
        Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'),
        HashRef()
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentsClient {
    my $self = shift;
    my ($hGetResponse, $hPostResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsClient');
    $Mockify->mock('doGetRequest')->when()->thenReturn($hGetResponse);
    $Mockify->mock('doPostRequest')->when()->thenReturn($hPostResponse // 1);
    $Mockify->mock('withTestMode')->when(Number())->thenReturn($Mockify->getMockObject());

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'ID' => 123
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'instanceOf' => 'CustomerOrder'
    };
    my $MockifyOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::Basket', $hAttributes
    );

    return $MockifyOrder->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Parent'     => $self->_createOrder(),
        'GrandTotal' => 0
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLineItemPaymentMolliePayments {
    my $self = shift;
    my ($hAttributes) = @_;

    my $LineItemContainer = $self->_createLineItemContainer();

    $hAttributes->{'PaymentMethod'} = $self->_createPaymentMethod();
    $hAttributes->{'LineItemContainer'} = $LineItemContainer;
    $hAttributes->{'Parent'} = $LineItemContainer;
    $hAttributes->{'instanceOf'} = 'LineItemPaymentMolliePayments';
    $hAttributes->{'IsTestTransaction'} = 1;
    $hAttributes->{'CurrencyID'} = 'EUR';

    $LineItemContainer->set({'GrandTotal' => $hAttributes->{'Amount'}});



    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
