#========================================================================================
# package      DE_EPAGES::PayPalExpress::Hooks::LineItemPaymentPayPalExpress
# state        public
# description  sets the ShippingAddressChanged flag
#========================================================================================
package DE_EPAGES::PayPalExpress::Hooks::LineItemPaymentPayPalExpress;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Order::API::Basket qw ( GetExistingBasket );
use DE_EPAGES::PayPalExpress::API::Payment qw ( DoCheckoutPayment );
use DE_EPAGES::PayPalExpress::API::Constants  qw ( TRANSSTATUS_FAILED );
#========================================================================================
# function     OnUpdateAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnUpdateAddress ($hParams);
#----------------------------------------------------------------------------------------
# description  Sets the ShippingAddressChanged flag if the shipping address was updated
#               (Hook OBJ_AfterUpdateAddress)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - address - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnUpdateAddress {
    my ($hParams) = @_;
    my $Address   = $hParams->{'Object'};
    my $Basket;

    if ($Address->alias =~ /^Shipping/i) {
        if ($Address->parent->instanceOf('Basket')){
            $Basket = $Address->parent;
        } else {
            my $User;
            if ($Address->parent->parent->instanceOf('AnonymousUser') ||
                $Address->parent->parent->instanceOf('User')) {
                $User = $Address->parent->parent;
            }
            else {
                return;
            }
            my $Shop = $Address->getSite();
            $Basket = GetExistingBasket($User, $Shop);
        }
        return unless defined $Basket;

        my $Container = $Basket->container;
        return unless defined $Container;

        my $PaymentLineItem = $Container->get('Payment');
        return unless defined $PaymentLineItem;

        my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
        return unless (defined($PaymentMethod) && $PaymentMethod->instanceOf('PaymentMethodPayPalExpress'));

        if ($PaymentLineItem->class->existsAttribute('ShippingAddressID')) {;
            my $ShippingAddressID = $PaymentLineItem->get('ShippingAddressID');
            if (!defined($ShippingAddressID) || $ShippingAddressID == $Address->id) {
                $PaymentLineItem->set({'ShippingAddressChanged' => 1});
            }
        }
    }
    return;
}

#========================================================================================
# function     OnUpdateBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnUpdateBasket ($hParams);
#----------------------------------------------------------------------------------------
# description  Sets the ShippingAddressChanged flag if another shipping address was selected
#               (Hook OBJ_AfterUpdateBasket)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - basket - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnUpdateBasket {
    my ($hParams) = @_;
    my $Basket    = $hParams->{'Object'};

    my $ShippingAddress = $Basket->get('ShippingAddress');
    return unless defined $ShippingAddress;

    my $Container = $Basket->container;
    return unless defined $Container;

    my $Payment = $Container->get('Payment');
    return unless defined $Payment;

    my $PaymentMethod = $Payment->get('PaymentMethod');
    return unless defined $PaymentMethod and $PaymentMethod->instanceOf('PaymentMethodPayPalExpress');

    my $ShippingAddressID = $Payment->get('ShippingAddressID');
    if (defined($ShippingAddressID) && $ShippingAddressID != $ShippingAddress->id) {
        $Payment->set ({'ShippingAddressChanged' => 1});
    }
}

#========================================================================================
# function     OnOrderCreated
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreated ($hParams);
#----------------------------------------------------------------------------------------
# description  triggers step 3 of PayPalExpress Payment Process: DoCheckoutPayment
#               (Hook API_OrderCreated)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - order - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderCreated {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};
    my $LineItemContainer = $Order->container();
    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless $LineItemPayment;
    return unless $LineItemPayment->instanceOf('LineItemPaymentPayPalExpress');

    # DoCheckoutPayment only if CheckoutRequestDetails happened before
    my $APIStep = $LineItemPayment->get('Field1');
    return unless (defined $APIStep and $APIStep eq 'CheckoutRequestDetails');

    my $hResult;
    eval {
        $hResult = DoCheckoutPayment($LineItemPayment, { 'ObjectID' => $Order->id() });
    };
    if (ExistsError()) {
        my $Error = GetError();

        if ($Error->code eq 'SEND_MAIL_FAILED'){
            Warning($Error);
        }
        elsif ($Error->code() eq 'PayPalExpressFailed') {
            unless ($Error->vars()->{'ErrorCode'} eq '10486') {
                $LineItemPayment->setTransactionTo(TRANSSTATUS_FAILED, $Error->vars()->{'ErrorCode'} );
            }
            #This Error is used to do a redirect to the SF from a Hook without servlet context
            Error('INVALID_FORM', {
                'ViewVars' => {
                    'FormError'           => 0,
                    'PayPalExpressFailed' => 1,
                    'ErrorCode'           => 'PayPalExpressInvalid',
                    'CorrelationID'       => $Error->vars()->{'CorrelationID'},
                    'PPEErrorCode'        => $Error->vars()->{'ErrorCode'},
                    'PayPalMessage'       => $Error->vars()->{'LongMessage'}
                },
                'ErrorObjectID' => $Order->id(),
                'ViewAction'    => 'ViewPayPalExpressFailed'
            });
        }
        else {
            Error($Error);
        }
    }
    elsif (defined($hResult) && defined($hResult->{'REDIRECTREQUIRED'}) && lc($hResult->{'REDIRECTREQUIRED'}) eq 'true') {
        #--- giropay redirect if necessary
        LogPayment('PayPalExpress', 'REDIRECTREQUIRED is true, call PayPalExpressGiroPayRedirect', {
            'OrderID'   => $Order->id(),
            'OrderPath' => $Order->pathString()
        });
        Error('INVALID_FORM', {
            'ViewVars' => {
                'FormError' => 0,
            },
            'ErrorObjectID' => $Order->id(),
            'ViewAction'    => 'PayPalExpressGiroPayRedirect'
        });
    }
}

#========================================================================================
# function     OnIsFraud
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnIsFraud($hParams);
#----------------------------------------------------------------------------------------
# description  if the payment was (partially) refunded, don't treat the order as fraud
#               (Hook CustomerOrderIsFraud)
#----------------------------------------------------------------------------------------
# input        $hParams | trigger parameter hash, keys are
#               <ul>
#                   <li>IsFraud - fraud status of the order - boolean</li>
#                   <li>LineItemPayment - payment line item of the order- object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnIsFraud {
    my ($hParams) = @_;
    my $LineItemPayment = $hParams->{'LineItemPayment'};

    # EPG-29351: don't treat orders with a (partial) refund as fraud
    if ($LineItemPayment->class->alias eq 'LineItemPaymentPayPalExpress'
            and defined $LineItemPayment->get('TransStatus')
            and $LineItemPayment->get('TransStatus') ~~ ['Refunded', 'PartiallyRefunded']) {
        $hParams->{'IsFraud'} = 0;
    }
}

1;
