#========================================================================================
# package      DE_EPAGES::ShopMigration::Hooks::Servlet
# state        public
# description  handles prepshop no login form
#========================================================================================
package DE_EPAGES::ShopMigration::Hooks::Servlet;
use base qw( Exporter );

use strict;
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::ShopMigration::API::Constants qw ( $UNAVAILABLE_VIEWACTIONS_MAP );

#========================================================================================
# function     OnBeforeHandlePermissionDenied
#----------------------------------------------------------------------------------------
# syntax       OnBeforeHandlePermissionDenied( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook UI_BeforeHandlePermissionDenied
#               Check if it is a prepshop and no user is logged in
#               if so, set new loginviewaction to UnityViewPrepShopLogin
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeHandlePermissionDenied {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    #return if user is logged in
    my $User = $Servlet->user;
    return if defined $User;

    return unless ($Servlet->_isAdministrativRequest ); ##no critic(ProhibitPrivateSubroutineUse)
    return if $Servlet->_isDiagnosticsRequest;          ##no critic(ProhibitPrivateSubroutineUse)

    my $Object = $Servlet->object;
    return unless defined $Object;

    my $Site = $Object->getSite();
    return unless $Site->instanceOf('Shop');
    # only do this for prep shops
    return unless $Site->get('IsMigrationPrepShop');
    #set new loginviewaction -> this is without form
    GetLog->debug('set loginviewaction UnityViewPrepShopLogin.');
    $hParams->{'LoginViewAction'} = 'UnityViewPrepShopLogin';
    return;
}

#========================================================================================
# function OnBeforeViewAction
#----------------------------------------------------------------------------------------
# syntax       OnBeforeViewAction( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook UI_BeforeViewAction
#               Checks for ViewActions of functionalities, which are not available in the prepshop.
#               if so, set the Viewaction to UnityMBO-ViewCannotEditInfoPage
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeViewAction {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $ViewAction = $Servlet->{'Params'}->{'ViewAction'};
    return unless exists $UNAVAILABLE_VIEWACTIONS_MAP->{$ViewAction};
    return unless $Servlet->object->getSite()->get('IsMigrationPrepShop');

    my $TranslatedFeatureName = Translate($UNAVAILABLE_VIEWACTIONS_MAP->{$ViewAction}->{translationKey},
        undef,
        $UNAVAILABLE_VIEWACTIONS_MAP->{$ViewAction}->{translationCartridge}
    );
    my $MBOMenuEntry = $UNAVAILABLE_VIEWACTIONS_MAP->{$ViewAction}->{mboMenuEntry};
    my $SubMenuEntry = $UNAVAILABLE_VIEWACTIONS_MAP->{$ViewAction}->{mboSubMenuEntry};
    my $HelpCenterLink = $UNAVAILABLE_VIEWACTIONS_MAP->{$ViewAction}->{helpCenterLink};
    $Servlet->vars('ViewAction', 'UnityMBO-ViewCannotEditInfoPage');
    $Servlet->vars('FeatureName', $TranslatedFeatureName);
    $Servlet->vars('MBOMenuEntry', $MBOMenuEntry);
    $Servlet->vars('MBOSubMenuEntry', $SubMenuEntry);
    $Servlet->vars('HelpCenterLink', $HelpCenterLink);
}

1;
