#========================================================================================
# package      DE_EPAGES::Tax::Hooks::ShopWizard
# state        public
# description  Sets the tax matrix for the shop by the selected ShopCountry.
#========================================================================================
package DE_EPAGES::Tax::Hooks::ShopWizard;

use strict;
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Tax::API::Constants qw( $SHOPWIZARD_TAXMATRIX_MAPPING );

#========================================================================================
# function     OnAfterFinishShopWizard
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterFinishShopWizard( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterFinishShopWizard is triggered, to set tax matrix on the shop
#               by the selected shop country. 
#               Triggered by Hook UI_FinishShopWizard
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Servlet
#                   <li>Data
#                   <li>Shop
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterFinishShopWizard {
    my ($hParams) = @_;
 
    my $Shop = $hParams->{'Shop'};

    # test if we have unity and if the attribute for changing matrix by country is activated
    my $hShopInfo = $Shop->get([qw(IsUnityRegistered CanSetTaxMatrixByShopCountry TaxMatrix)]);

    return unless $hShopInfo->{'IsUnityRegistered'};
    return unless $hShopInfo->{'CanSetTaxMatrixByShopCountry'};

    my $Store = LoadRootObject();
    # prevent changing an adapted tax matrix (this should normally not happen)
    # test if the shop tax matrix is not the system tax matrix object
    my $ShopTaxMatrix = $hShopInfo->{'TaxMatrix'};
    return unless $ShopTaxMatrix->id eq $Store->child($ShopTaxMatrix->alias)->id;

    # get the country from the shop address
    my $Address = $Shop->get('Address');
    my $EnglishCountry = $Address->get('Country')->{'English'};
    return unless $EnglishCountry;

    # we do not set the matrix if we change the address to US (TaxUS is used)
    return if $EnglishCountry eq 'United States';

    # test if we have to map the country name to an other name
    # (can also be used to define different defaults for countries)
    my $CountryPostFix = $EnglishCountry;
    if(defined $SHOPWIZARD_TAXMATRIX_MAPPING->{$EnglishCountry} ){
        $CountryPostFix = $SHOPWIZARD_TAXMATRIX_MAPPING->{$EnglishCountry};
    }

    # test if we have a tax matrix on the system for the country
    # test also the instances of the object
    my $TaxMatrixAlias = 'TaxMatrix'. $CountryPostFix;

    # Add exception for shops in Northern Ireland.
    # These shop have to use the NI tax matrix because NI is still in the EU.
    # We test here only the country id and IsEU. This is valid for NI because
    # IsEU returns 1 if we have a UK CountryID (826) and NI zip codes (BT01-BT94)
    #(see DE_EPAGES::Shop::API::Address::IsEUAddress)
    if ($Address->get('CountryID') eq '826' and $Address->get('IsEU')) {
        my $NITaxMatrixAlias = 'TaxMatrixNorthernIreland';
        # test if we have a matrix for NI on the system
        # otherwise use the suggested
        if($Store->existsChild($NITaxMatrixAlias)){
            $TaxMatrixAlias = 'TaxMatrixNorthernIreland';
        }
    }

    if ($Store->existsChild($TaxMatrixAlias)){
        my $SystemTaxMatrix = $Store->child($TaxMatrixAlias);
        return unless $SystemTaxMatrix->instanceOf('TaxMatrix'); 

        # now set the new matrix for the shop
        $Shop->set({ 'TaxMatrix' => $SystemTaxMatrix });
    }
}

1;
