use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use DE_EPAGES::Core::API::Error qw(GetError ExistsError);
use DE_EPAGES::Core::API::Log qw(LogError);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Database::API::Connection qw(RunOnStore);
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw(DeleteObjects);
use DE_EPAGES::Object::API::Table::Country qw (ExistsCountry);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);
use DE_EPAGES::Tax::API::Cartridge;


#========================================================================================
# function         _setupTaxMatrixDigitalForBrexit
# state            private
#----------------------------------------------------------------------------------------
# syntax           _setupTaxMatrixDigitalForBrexit();
#----------------------------------------------------------------------------------------
# description      Change Alias and CountryID for DigitalTaxMatrix UnitedKingdom to
#                   UnitedKingdomEU (Alias) and 000 (CountryID).
#========================================================================================
sub _setupTaxMatrixDigitalForBrexit {
    my ($FilesDir) = @_;

    my $TaxMatrixUK = LoadObjectByPath('/TaxMatrixDigital');

    if ($TaxMatrixUK->existsChild('UnitedKingdomEU')) {
        print "TaxMatrixDigital UnitedKingdomEU already exists";
        return;
    }
    eval {
        printf "Rename TaxMatrixDigital/UnitedKingdom to TaxMatrixDigital/UnitedKingdomEU and set new Country \n";
        ImportObjects($FilesDir.'/changeTaxMatrixDigitalUK.xml');
    };
    LogError(GetError()) if ExistsError();

    eval {SynchronizeCache();};
    LogError(GetError()) if ExistsError();

    return;
}

#========================================================================================
# function         _setupTaxMatrixGreatBritainForBrexit
# state            private
#----------------------------------------------------------------------------------------
# syntax           _setupTaxMatrixGreatBritainForBrexit();
#----------------------------------------------------------------------------------------
# description      Change Alias for EU and non-EU Taxareas for TaxMatrixGreatBritain to
#                   United Kingdom and abroad.
#========================================================================================
sub _setupTaxMatrixGreatBritainForBrexit {
    my ($FilesDir) = @_;
    eval {
        my $TaxMatrixUK = LoadObjectByPath('/TaxMatrixGreatBritain');

        if ($TaxMatrixUK->existsChild('abroad')) {
            print "TaxArea abroad already exists for TaxMatrixGreatBritain\n";
            return;
        }
        print "Rename TaxArea EU to United Kingdom and non-EU to abroad \n";
        ImportObjects($FilesDir.'/renameTaxAreasGreatBritain.xml');

        print "remove EU countries from United Kingdom TaxArea in TaxMatrixGreatBritain \n";
        DeleteObjects($FilesDir.'/removeGreatBritainfromEU.xml');

        print "Import EU exceptions for Northern Ireland \n";
        ImportObjects($FilesDir.'/importTaxAreaExeptionsNorthIreland.xml');

    };
    LogError(GetError()) if ExistsError();

    eval {SynchronizeCache();};
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless (defined $StoreName) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {

            my $FilesDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Tax/Scripts/Brexit_files";

            if (!ExistsCountry('000')) {
                print "\nThe fake United Kingdom (EU) country does not exists. Import now Brexit_files/imp_Country_000.xml \n";
                ImportObjects($FilesDir.'/imp_Country_000.xml');
            }
                _setupTaxMatrixDigitalForBrexit($FilesDir);
                _setupTaxMatrixGreatBritainForBrexit($FilesDir);
        },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Make changes for TaxMatrixGreatBritain  and TaxMatrixdigital
    necessary beacuse of Brexit on 01.01.2021.
    Changes for TaxMatrixDigital are:
        1. Rename TaxMatrixDigital/UnitedKindom to UnitedKingdomEU
        2. Set CoutnryID to 000 for TaxMatrixDigital/UnitedKindomEU

    Changes for TaxMatrixGreatBritain are:
        1. Rename TaxArea EU to United Kingdom
        2. Rename Taxarea non-EU to abroad
        3. Remove EU countries from United Kingdom TaxArea
Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);