use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use DE_EPAGES::Address::API::Address qw(CompareZipCodes);
use DE_EPAGES::Core::API::Error qw(GetError ExistsError);
use DE_EPAGES::Core::API::Log qw(LogError);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Database::API::Connection qw(RunOnStore);
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);
use DE_EPAGES::Tax::API::Cartridge;


#========================================================================================
# function         _setupTaxMatrixForStayingNIinTheEU
# state            private
#----------------------------------------------------------------------------------------
# syntax           _setupTaxMatrixForStayingNIinTheEU();
#----------------------------------------------------------------------------------------
# description      Change Alias and CountryID for DigitalTaxMatrix UnitedKingdom to
#                   UnitedKingdomEU (Alias) and 000 (CountryID).
#========================================================================================
sub _setupTaxMatrixForStayingNIinTheEU {
    my ($ShopName) = @_;

    my $System = LoadRootObject();
    my $Cartridge = DE_EPAGES::Tax::API::Cartridge->new;

    # import new Matrix TaxMatrixNorthernIreland (NI) if not already exists
    unless ($System->existsChild('TaxMatrixNorthernIreland')){
        ImportObjects($Cartridge->xmlDirectory().'/TaxesNorthernIreland.xml');
    }

    my $NITaxMatrix = $System->child('TaxMatrixNorthernIreland');
    my $NITaxMatrixAlias = $NITaxMatrix->alias;
    my $UKTaxMatrixID = $System->child('TaxMatrixGreatBritain')->id;
    my $UKCountryID = '826';
    my $ZipFrom = 'BT01';
    my $ZipTo   = 'BT94';

    my $aShops = defined $ShopName ? [LoadObjectByPath('/Shops')->child($ShopName)] : $System->get('Shops');

    # assign TaxMatrixNorthernIreland to NI shops (TaxMatrixGreatBritain and having zip code ranges BT01 - BT99)
    eval {
        foreach my $Shop (@$aShops) {
            # next if shop has not the UK tax matrix from the system
            # (ignore adapted tax matrix and other)
            next unless $Shop->get('TaxMatrix')->id eq $UKTaxMatrixID;

            # find out if we have an NI shop with the correct countryid ant the zip code range
            my $ShopAddress = $Shop->get('Address');
            next unless defined $ShopAddress;

            next unless $ShopAddress->get('CountryID') eq $UKCountryID;
            my $ZipCode = $ShopAddress->get('Zipcode');

            next unless $ZipCode;

            if ((CompareZipCodes($ZipFrom, $ZipCode) != 1) and 
                (CompareZipCodes($ZipTo, $ZipCode) != -1)){

                print "set Shop: " . $Shop->alias . " to new TaxMatrix: $NITaxMatrixAlias\n";
                $Shop->set({'TaxMatrix' => $NITaxMatrix});

            }
        }
    };

    LogError(GetError()) if ExistsError();

    eval {SynchronizeCache();};
    LogError(GetError()) if ExistsError();

    return;
}


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopname=s'  => \$ShopName
    );

    usage() if $Help;

    unless (defined $StoreName) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
                _setupTaxMatrixForStayingNIinTheEU($ShopName,);

        },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds a new tax matrix for shops in northern ireland for having NI still in the EU
    (ignore shops with an adapted tax matrix)

    - creates a new tax matrix for NI on the system (TaxMatrixNorthernIreland) if not exists by importing TaxesNorthernIreland.xml
    - find out shops located in NI by testing the existing tax matrix, using the countryid from uk and the zip code range BT01-BT94
    - set the new tax matrix to the shop if not already done

    (This script can run multiple times for correcting also shops afterwards if a merchant resets his adaption)


Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopname   name of shop (optional)                         ()
flags:
    -help       show the command line options
Example:
    perl $0 -storename Store -shopname MyNiceShop
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);