#========================================================================================
# package      DE_EPAGES::Test::API::MySQL
# base         DE_EPAGES::Test::API::StoreTypeUnitTest
# state        public
#----------------------------------------------------------------------------------------
# description  Tests for *.sql correctness
#========================================================================================
package DE_EPAGES::Test::API::MySQL;
use base DE_EPAGES::Test::API::UnitTest;

use strict;
use Getopt::Long;
use Test::More;
use DE_EPAGES::Core::API::Error qw (VarsOnError);
use DE_EPAGES::Core::API::Git;
use DE_EPAGES::Object::API::SQLStructureImportHandler;
use DE_EPAGES::Object::API::Object::SQLStructure;
use DE_EPAGES::Installer::API::GitTag;
use File::Basename qw (dirname basename);
use DE_EPAGES::Core::API::File qw (
    ExistsFile
    ExistsDirectory
    GetFileNames
    GetSubDirectories
    RunOnDirectory
    CopyFile
    CreateDirectory
    DeleteDirectory
);
use Cwd;

sub test {
    my $self = shift;

    my $Help;
    my $Company = 'DE_EPAGES';

    # Create temp directory
    my $CurrentDir = getcwd();
    my $TemporaryWorkingDir = $CurrentDir . '/temp';
    $self->{'TemporaryWorkingDir'} = $TemporaryWorkingDir;

    GetOptions(
        'help'          => \$Help,
        'company=s'     => \$Company
    );
    usage() if $Help;
    my $CheckAll = lc($Company) eq 'all' ? 1 : 0;
    my $WorkingDir = $ENV{'EPAGES_CARTRIDGES'};
    my @SQLDirMap;

    #fill @SQLDirMap with all mysql folders from ../Cartridges
    if ($CheckAll) {
        foreach my $CompanyFolder (@{GetSubDirectories($WorkingDir)}) {
            foreach my $CartridgeFolder (@{GetSubDirectories($CompanyFolder)}){
                push (@SQLDirMap,_GetSqlDirectory($CartridgeFolder)) if (_ExistsSqlDirectory($CartridgeFolder));
            }
        }
    }
    else {
        my $CompanyFolder = $WorkingDir . '/' . $Company;

        foreach my $CartridgeFolder (@{GetSubDirectories($CompanyFolder)}){
            push (@SQLDirMap,_GetSqlDirectory($CartridgeFolder)) if (_ExistsSqlDirectory($CartridgeFolder));
        }
    }

    # Get *.sql files and not look in delete_...sql files (structure importhandler does not support DROP statements).
    my @SQLFiles;
    foreach my $SQLDir (@SQLDirMap) {
        push @SQLFiles, grep { $_ =~ m/\.sql$/ && $_ !~ m/delete_.*?\.sql$/ } @{GetFileNames($SQLDir, {'recursive' => 1})};
    }

    # Get sources
    _CheckoutSqlSources($TemporaryWorkingDir);

    # Test each sql file.
    foreach my $SQLFile (@SQLFiles) {
        # Check if index columns have not desc or asc sortorder.
        CheckIndexColumnSortOrder($SQLFile);

        #check for zero downtime restrictions
        CheckZeroDownTimeRestrictions($SQLFile, $TemporaryWorkingDir);
    }

    ok(1, 'all tests executed');

    done_testing();
}

sub cleanup {
    my $self = shift;

    DeleteDirectory($self->{'TemporaryWorkingDir'}, {'recursive' => 1});
}

#========================================================================================
# function     CheckIndexColumnSortOrder
#----------------------------------------------------------------------------------------
# syntax       CheckIndexColumnSortOrder($SQLFile);
#----------------------------------------------------------------------------------------
# description  Check if index columns have not desc or asc sortorder.
#----------------------------------------------------------------------------------------
# input        $SQLFile | sql files | string
#========================================================================================
sub CheckIndexColumnSortOrder {
    my ($SQLFile) = @_;

    # Import sql file.
    my $SQLStructure = eval { _ImportSQL($SQLFile) };
    VarsOnError( { SQLFile => $SQLFile } );

    # Loop over tables.
    foreach my $Table (@{$SQLStructure->getTables()}) {
        # Loop over table indeces.
        foreach my $Index (@{$SQLStructure->getTableIndices($Table)}) {
            # Loop over index columns.
            foreach my $IndexColumn (@{$SQLStructure->getTableIndexColumns($Table, $Index)}) {
                # Get sort order.
                my $SortOrder = $SQLStructure->getTableIndexColumnSortOrder($Table, $Index, $IndexColumn);

                #sort order must not be DESC or ASC
                if ( $SortOrder ~~ [qw(DESC ASC)] ) {
                    ok(
                        0,
                        "index column must not have sort order '$SortOrder': file='$SQLFile', table='$Table', index='$Index', column='$IndexColumn'"
                    );
                }
            }
        }
    }
}

#========================================================================================
# function     CheckZeroDownTimeRestrictions
#----------------------------------------------------------------------------------------
# syntax       CheckZeroDownTimeRestrictions($SQLFile);
#----------------------------------------------------------------------------------------
# description  Check if some changes to previous release blocking zero down time.
#               returns true if changes are zero down time conform
#----------------------------------------------------------------------------------------
# input        $SQLFile | sql files | string
# input        $TemporaryWorkingDir | folder which will contain temporary copies of SQL files. | string
#========================================================================================
sub CheckZeroDownTimeRestrictions {
    my ($SQLFile, $TemporaryWorkingDir) = @_;

    my $Dir = dirname($SQLFile);
    my $LocalFileName = basename($SQLFile);
    my $OldLocalFileName = 'old_' . $LocalFileName;
    my $NewFile = $TemporaryWorkingDir . '/new_' . $LocalFileName;
    my $OldFile = $TemporaryWorkingDir . '/' . $OldLocalFileName;
    my $SQLStructImportHandler = DE_EPAGES::Object::API::SQLStructureImportHandler->new;
    my $CurrentSQLFile = $SQLFile;
    $CurrentSQLFile =~ s/$ENV{EPAGES_CARTRIDGES}\///;

    #get system patch builder
    my $GitTag = DE_EPAGES::Installer::API::GitTag->new();
    my $PreviousReleaseTag = $GitTag->GetPreviousGitReleaseTag();

    #quick fix for EPG-35893
    #TODO: take out if problem fixed with installer check in function 'test'
    return unless ( ExistsFile($TemporaryWorkingDir . '/OLD_VERSION/'.$CurrentSQLFile) );

    #run on directory
    RunOnDirectory(
        $TemporaryWorkingDir,
        sub {
            CopyFile('HEAD/'.$CurrentSQLFile, $SQLFile);
            CopyFile('OLD_VERSION/'.$CurrentSQLFile, $OldFile);
        }
    );

    #run on directory
    RunOnDirectory(
        $Dir,
        sub {
            CopyFile($SQLFile, $NewFile, {'create_directory' => 1});
        }
    );

    #get SQL-Structures
    my $NewSQLStruct = $SQLStructImportHandler->getSQLStructure($NewFile);
    my $OldSQLStruct = $SQLStructImportHandler->getSQLStructure($OldFile);

    #zdt check using sql structures
    _CheckSQLStructsForZeroDT($NewSQLStruct,$OldSQLStruct, $SQLFile, $PreviousReleaseTag);

    return;
}

sub _altTableName {
  # returns either NAME or `NAME`
  my ($Table, $SQLStruct) = @_;
  return $Table if $SQLStruct->existsTable($Table);

  my $AltTable = "\`$Table\`";
  $AltTable =~ s,\`,,g if $Table =~ /\`/;
  return $SQLStruct->existsTable($AltTable) ? $AltTable : $Table;
}

sub _CheckSQLStructsForZeroDT{
    my ($NewSQLStruct,$OldSQLStruct, $SQLFilePath, $PreviousReleaseTag) = @_;

    #
    foreach my $Table (@{$OldSQLStruct->getTables()}) {
        my $NewTable = _altTableName($Table, $NewSQLStruct);
        # check for deleted column
        foreach my $Column (@{$OldSQLStruct->getTableColumns($Table)}) {
            unless ($NewSQLStruct->existsTableColumn($NewTable,$Column)){
                ok(
                    0,
                    "zero down time check for missing column '$Table.$Column', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                );
            }
            else{
                #get table column info
                my $hOldTableColumnInfo = $OldSQLStruct->getTableColumnInfo($Table, $Column);
                my $hNewTableColumnInfo = $NewSQLStruct->getTableColumnInfo($NewTable, $Column);

                #check if column definition is the same
                unless (
                    _CompareHashes(
                        $hOldTableColumnInfo,
                        $hNewTableColumnInfo
                    )
                ){
                    #transform from not null to null is allowed
                    if (($hOldTableColumnInfo->{'NotNull'} eq '1') and ($hNewTableColumnInfo->{'NotNull'} eq '0')){
                        delete $hOldTableColumnInfo->{'NotNull'};
                        delete $hNewTableColumnInfo->{'NotNull'};
                    }

                    #check info to be equal
                    is_deeply(
                        $hOldTableColumnInfo,
                        $hNewTableColumnInfo,
                        "zero down time check for column definition '$Table.$Column', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                    );
                }
            }
        }

        ##check for primary key name
        #ok (
        #    ($NewSQLStruct->getPrimaryKeyName($Table) eq $OldSQLStruct->getPrimaryKeyName($Table)),
        #    "zero down time check for primary key name for table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
        #);

        #check for primary key columns
        unless(_CompareArrays($NewSQLStruct->getPrimaryKeyColumns($NewTable),
                              $OldSQLStruct->getPrimaryKeyColumns($Table))){
            is_deeply (
                $NewSQLStruct->getPrimaryKeyColumns($NewTable),
                $OldSQLStruct->getPrimaryKeyColumns($Table),
                "zero down time check for primary key columns for table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
            );
        }

    }

    #check for new not null columns
    foreach my $Table (@{$NewSQLStruct->getTables()}) {
        my $OldTable = _altTableName($Table, $OldSQLStruct);
        # Loop over table columns
        foreach my $Column (@{$NewSQLStruct->getTableColumns($Table)}) {
            unless($OldSQLStruct->existsTableColumn($OldTable,$Column)){
                #check if no new table column exists with not null
                if ($NewSQLStruct->getTableColumnNotNull($Table,$Column)){
                    ok(
                        0,
                        "zero down time check for new 'not null' column '$Table.$Column', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                    );
                }
            }
        }
    }

    #check foreign keys of old structure
    foreach my $Table (@{$OldSQLStruct->getTables()}) {
        my $NewTable = _altTableName($Table, $NewSQLStruct);
        # Loop over table foreign keys
        foreach my $ForeignKey (@{$OldSQLStruct->getTableForeignKeys($Table)}) {
            #check if foreign key is deleted
            unless ($NewSQLStruct->existsTableForeignKey($NewTable,$ForeignKey)){
                ok(
                    0,
                    "zero down time check for deleted foreign key '$ForeignKey' in table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                );
            }
            else{
                #check foreign key info
                my $OldRef = $OldSQLStruct->getTableForeignKeyRefTable($Table, $ForeignKey);
                my $NewRef = $NewSQLStruct->getTableForeignKeyRefTable($NewTable, $ForeignKey);
                $OldRef =~ s,\`,,g; # table is either NAME or `NAME`
                $NewRef =~ s,\`,,g; # table is either NAME or `NAME`
                unless($OldRef eq $NewRef){
                    ok(
                        0,
                        "zero down time check for foreign key '$ForeignKey' reference table in table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                    );

                }

                unless (
                    _CompareHashes(
                        $OldSQLStruct->getTableForeignKeyColumnPairs($Table, $ForeignKey),
                        $NewSQLStruct->getTableForeignKeyColumnPairs($NewTable, $ForeignKey)
                    )
                ){
                    is_deeply(
                        $OldSQLStruct->getTableForeignKeyColumnPairs($Table, $ForeignKey),
                        $NewSQLStruct->getTableForeignKeyColumnPairs($NewTable, $ForeignKey),
                        "zero down time check for foreign key column pairs '$ForeignKey' reference table in table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                    );
                }
            }
        }
    }

    #check for indices
    foreach my $Table (@{$OldSQLStruct->getTables()}){
        my $NewTable = _altTableName($Table, $NewSQLStruct);
        #loop over indizes
        foreach my $Index (@{$OldSQLStruct->getTableIndices($Table)}){

            #check for deleted indizes
            unless($NewSQLStruct->existsTableIndex($NewTable,$Index)){
                ok(
                    0,
                    "zero down time check for missing index '$Index' in table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                );
            }
            else{

                #check for index columns
                unless(_CompareArrays($OldSQLStruct->getTableIndexColumns($Table, $Index),
                                      $NewSQLStruct->getTableIndexColumns($NewTable, $Index))){
                    is_deeply(
                        $OldSQLStruct->getTableIndexColumns($Table, $Index),
                        $NewSQLStruct->getTableIndexColumns($NewTable, $Index),
                        "zero down time check for index columns '$Index' in table '$Table', previous release '$PreviousReleaseTag', file '$SQLFilePath'"
                    );
                }
            }
        }
    }
}

#----------------------------------------------------------------------------------------
#  private helper
#----------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------
# Checkout Cartridges DE_EPAGES from Git for HEAD and PreviousReleaseTag
#----------------------------------------------------------------------------------------
sub _CheckoutSqlSources {
    my ($TemporaryWorkingDir) = @_;

    DeleteDirectory($TemporaryWorkingDir,{'recursive' => 1});
    CreateDirectory($TemporaryWorkingDir);

    #get git
    my $Git = DE_EPAGES::Core::API::Git->new();
    my $GitTag = DE_EPAGES::Installer::API::GitTag->new();

    #get previous version
    my $PreviousReleaseTag = $GitTag->GetPreviousGitReleaseTag();

    #create sub folders for previous version
    CreateDirectory($TemporaryWorkingDir.'/OLD_VERSION');

    if(ExistsDirectory("$ENV{EPAGES_CARTRIDGES}/.git")) {
        #checkout sources for previous version and HEAD in $TemporaryWorkingDir
        CreateDirectory($TemporaryWorkingDir.'/HEAD');
        $Git->GitExportRepo('develop', $ENV{EPAGES_CARTRIDGES}, "$TemporaryWorkingDir/HEAD");
        $Git->GitExportRepo($PreviousReleaseTag, $ENV{EPAGES_CARTRIDGES}, "$TemporaryWorkingDir/OLD_VERSION");
    } else {
        $Git->CloneRepository('git@github.com:ePages-de/Cartridges.git', "$TemporaryWorkingDir/HEAD");

        #checkout sources for previous version in $TemporaryWorkingDir.'/OLD_VERSION'
        $Git->GitExportRepo($PreviousReleaseTag, "$TemporaryWorkingDir/HEAD", "$TemporaryWorkingDir/OLD_VERSION");
    }
}

#----------------------------------------------------------------------------------------
# Get sql directory of a cartridge
#----------------------------------------------------------------------------------------
sub _GetSqlDirectory {
    my ($Directory) = @_;

    return $Directory.'/Database/mysql/Tables'
}

#----------------------------------------------------------------------------------------
# Check if sql directory of a cartridge exists.
#----------------------------------------------------------------------------------------
sub _ExistsSqlDirectory {
    my ($Package) = @_;

    return ExistsDirectory(_GetSqlDirectory($Package));
}

sub _CompareArrays{
    my ($aArray1, $aArray2) = @_;

    if (scalar (@$aArray1) != scalar (@$aArray2)) {
        return 0;
    }
    else {
        foreach (my $i = 0; $i < scalar (@$aArray1); $i++) {
            #check values
            if ($aArray1->[$i] ne $aArray2->[$i]) {
                return 0;
            }
        }
    }

    return 1;
}

sub _CompareHashes{
    my ($hHash1, $hHash2) = @_;

    if (scalar (keys %$hHash1) != scalar (keys %$hHash2)){
        return 0;
    }
    else {
        foreach my $Key (keys %$hHash1) {
            unless (defined $hHash2->{$Key}){
                return 0;
            }
            else{
                unless ($hHash1->{$Key} eq $hHash2->{$Key}) {
                    return 0;
                }
            }
        }
    }

    return 1;
}

#----------------------------------------------------------------------------------------
# Import sql file.
#----------------------------------------------------------------------------------------
sub _ImportSQL {
    my ($FileName) = @_;

    # Get and return SQL structure object from SQL file.
    my $Importer = DE_EPAGES::Object::API::SQLStructureImportHandler->new;
    return $Importer->getSQLStructure($FileName);
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] flags objects

options:
    -company    default "DE_EPAGES", "all" checks all folders in Cartridges directory (optional)
flags:
    -help       show the command line options

Example:
    perl $0 -company all
    perl $0 -company ES_CONSULTING
    perl $0 -company DE_EPAGES
END_USAGE
    exit 2;
}

1;
